@@ -76,7 +76,7 @@ NNS.reg(iris[ , 1 : 4], iris[ , 5], type = "CLASS", point.est = iris[1 : 10, 1 :
 #  [1] 1
 #  
 #  $NNS.reg.n.best
-#  [1] 1
+#  [1] 4
 #  
 #  $probability.threshold
 #  [1] 0.43875
@@ -128,5 +128,5 @@ NNS.reg(cbind(x, x), y,
         ncores = 1, confidence.interval = .95)
 
 ## ----threads, echo = FALSE----------------------------------------------------
-Sys.setenv("OMP_THREAD_LIMIT" = 0)
+Sys.setenv("OMP_THREAD_LIMIT" = "")
 
